from typing import List, Optional, Self, Set, Union

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator, UiModToConfMap
from edge.webserver.sysconf.dgmod.gc.gc_main import MainConfBuilder


# noinspection PyUnusedLocal
def gc_get_creators(uimod: Union[ph.UiMod, ph.UiMod2],
                    all_used_uimods: Set[Union[ph.UiMod, ph.UiMod2]],
                    uimod_to_newconf_map: UiModToConfMap,
                    forced: bool = False) -> List[BaseCreator]:

    if uimod.type != ph.UI_TYPE_MV_CAMERA:
        return []

    try:
        cam_uimod = next(filter(lambda x: x.type == ph.UI_TYPE_MV_CAMERA, all_used_uimods))
        lpr_uimod = next(filter(lambda x: x.type == ph.UI_TYPE_LPR, all_used_uimods))

        activate = any([
            uimod_to_newconf_map.get(cam_uimod)
        ])

        if forced or activate:
            return [GcCreator(cam_uimod, lpr_uimod)]

    except StopIteration:
        pass

    return []


class GcCreator(BaseCreator):
    def __init__(self, cam_uimod: ph.UiMod, lpr_uimod: ph.UiMod) -> None:
        super().__init__()

        self._dgmod_gc = dg.DgMod.create(dg.EDGE_TYPE_GRABCONTROL, cam_uimod.pid, cam_uimod.id)

        self._uimod_cam = cam_uimod
        self._uimod_lpr = lpr_uimod

        self._main_active = None

    @property
    def dgmod(self) -> dg.DgMod:
        return self._dgmod_gc

    def prepare(self, uimod_to_newconf_map: UiModToConfMap) -> None:
        spool_dir = ph.get_spool_dir(self._uimod_cam.pid)

        cam_uiconf = uimod_to_newconf_map.get(self._uimod_cam) or self._uimod_cam.load_conf()
        cam_uiconf_control = cam_uiconf['rw']['videostream']['control']

        cam_uiconf_pri = cam_uiconf['ro']

        #
        exp_time_conf = [0, 0, 0]
        gain_conf = [0, 0, 0]
        gain_step = 1.0
        exptime_step = 20
        if cam_uiconf_control['profile'] == 'auto-70-150':
            gain_conf = [cam_uiconf_control['gain-auto-min'], cam_uiconf_control['gain-auto-max'], gain_step]
            exp_time_conf = [
                cam_uiconf_control['exposure-auto-min'],
                cam_uiconf_control['exposure-auto-max'],
                exptime_step]
        #

        main_builder = MainConfBuilder()
        main_builder.setup_ir_control(cam_uiconf_pri.get('control-ir', True))
        main_builder.setup_gain_conf(gain_conf)
        main_builder.setup_exp_time_conf(exp_time_conf)
        main_builder.setup_add_scene_img_react(spool_dir, f'mg{self._uimod_cam.id}-gc{self._uimod_cam.id}-scene')
        main_builder.setup_add_scene_aoi_react(spool_dir, f'lr{self._uimod_lpr.id}-gc{self._uimod_cam.id}-scene')
        main_builder.setup_add_scene_control_event(f'/etc/edge/mediagrabber.{self._dgmod_gc.id}/dyn/')

        main_conf = main_builder.build()
        ss_active = cam_uiconf_control['profile'] == 'auto-70-150'

        self._main_active = main_conf, ss_active

    def complete(self) -> None:
        main_conf, ss_active = self._main_active

        dgmod = self._dgmod_gc
        dgmod.install()
        dgmod.save_main_conf(main_conf)
        dgmod.save_setupstore_conf(ss_active)

    def rollback(self) -> None:
        pass

    def notify(self) -> None:
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class GcDeleter:
    def __init__(self, dgmod: dg.DgMod):
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod: ph.UiMod) -> Optional[Self]:
        if entry_uimod.type == ph.UI_TYPE_MV_CAMERA:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_GRABCONTROL, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self) -> None:
        self._dgmod.uninstall()
