from typing import List, Optional, Self, Set, Union

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator, UiModToConfMap
from edge.webserver.sysconf.dgmod.rc.rc_main import MainConfBuilder


# noinspection PyUnusedLocal
def rc_get_creators(entry_uimod: Union[ph.UiMod, ph.UiMod2],
                    all_used_uimods: Set[Union[ph.UiMod, ph.UiMod2]],
                    uimod_to_newconf_map: UiModToConfMap,
                    forced: bool = False) -> List[BaseCreator]:

    if entry_uimod.type != ph.UI_TYPE_MV_CAMERA:
        return []

    try:
        cam_uimod, lpr_uimod, fusion_uimod = (
            next(filter(lambda x: x.type == ph.UI_TYPE_MV_CAMERA, all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_LPR, all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_FUSION, all_used_uimods)))

        activate = any([
            uimod_to_newconf_map.get(cam_uimod)
        ])

        if forced or activate:
            return [RcCreator(cam_uimod, lpr_uimod, fusion_uimod)]

    except StopIteration:
        pass

    return []


class RcCreator(BaseCreator):
    def __init__(self, cam_uimod: ph.UiMod, lpr_uimod: ph.UiMod, fusion_uimod: ph.UiMod) -> None:
        super().__init__()

        self._dgmod_rc = dg.DgMod.create(dg.EDGE_TYPE_RAWCODEC, cam_uimod.pid, cam_uimod.id)

        self._uimod_cam = cam_uimod
        self._uimod_lpr = lpr_uimod
        self._uimod_fusion = fusion_uimod

        self._main_ss = None

    @property
    def dgmod(self) -> dg.DgMod:
        return self._dgmod_rc

    def prepare(self, uimod_to_newconf_map: UiModToConfMap) -> None:
        main_builder = MainConfBuilder(self._uimod_cam.id, ph.get_spool_dir(self._uimod_cam.pid))

        cam_uiconf = uimod_to_newconf_map.get(self._uimod_cam) or self._uimod_cam.load_conf()

        main_builder.setup_react('mg{0}-rc{0}-scene'.format(main_builder.uimod_id))
        main_builder.setup_event_blobs('rc{0}-rc{0}-store'.format(main_builder.uimod_id))
        main_builder.setup_event_args([
            f'rc{main_builder.uimod_id}-lr{self._uimod_lpr.id}-scene',
            f'rc{main_builder.uimod_id}-tr{self._uimod_fusion.id}-scene',
            'mg{0}-mcgst{0}-scene'.format(main_builder.uimod_id)
        ])

        ss_active = cam_uiconf['ro']['raw-format'] != 'MONO8T'
        if ss_active and cam_uiconf['rw']['videostream']['control']['profile'] == 'auto-70-150':
            main_builder.setup_event_args([
                'mg{0}-gc{0}-scene'.format(main_builder.uimod_id)
            ])

        main_conf = main_builder.build()
        self._main_ss = main_conf, ss_active

    def complete(self) -> None:
        main_conf, ss_active = self._main_ss

        dgmod = self._dgmod_rc
        dgmod.install()
        dgmod.save_main_conf(main_conf)
        dgmod.save_setupstore_conf(ss_active)

    def rollback(self) -> None:
        pass

    def notify(self) -> None:
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class RcDeleter:
    def __init__(self, dgmod: dg.DgMod):
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod: ph.UiMod) -> Optional[Self]:
        if entry_uimod.type == ph.UI_TYPE_MV_CAMERA:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_RAWCODEC, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self) -> None:
        self._dgmod.uninstall()
