from typing import Final, List, Optional, Self, Set, Union

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator, UiModToConfMap
from edge.webserver.sysconf.dgmod.rn.rn_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.rn.rn_setupstore import SetupStoreConfBuilder
from edge.webserver.sysconf.dgmod.rn.uiconf import IUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.camera import CameraUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.fusion import FusionUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.gnss import GnssUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.location import LocationUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.redlight import RedlightUiConfProvider, RailroadUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.roadway import RoadwayUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.spotspeed import SpotSpeedUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.streaming import HttpStreamingUiConfProvider, RtspStreamingUiConfProvider
from edge.webserver.sysconf.dgmod.rn.uiconf.webserver import WebServerUiConfProvider
from edge.webserver.sysconf.uiconf.time import TimeUiconf


# noinspection PyUnusedLocal
def or_get_creators(entry_uimod: Union[ph.UiMod, ph.UiMod2],
                    all_used_uimods: Set[Union[ph.UiMod, ph.UiMod2]],
                    uimod_to_newconf_map: UiModToConfMap,
                    forced: bool = False) -> List[BaseCreator]:

    if entry_uimod.type not in [ph.UI_TYPE_HTTP_STREAMING, ph.UI_TYPE_RTSP_STREAMING]:
        return []

    try:
        (ra_uimod, rstreaming_uimod, hstreaming_uimod, fusion_uimod, ws_uimod, term_uimod,
         gnss_uimod, cam_uimod, roadway_uimod, location_uimod) = (
            next(filter(lambda x: x.type == ph.UI_TYPE_RADAR, all_used_uimods), None),
            next(filter(lambda x: x.type == ph.UI_TYPE_RTSP_STREAMING, all_used_uimods), None),
            next(filter(lambda x: x.type == ph.UI_TYPE_HTTP_STREAMING, all_used_uimods), None),
            next(filter(lambda x: x.type == ph.UI_TYPE_FUSION, all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_WEBSERVER, all_used_uimods), None),
            next(filter(lambda x: x.type == ph.UI_TYPE_TERMINAL, all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_GNSS, all_used_uimods), None),
            next(filter(lambda x: x.type in [ph.UI_TYPE_MV_CAMERA, ph.UI_TYPE_RTSP_CAMERA], all_used_uimods), None),
            next(filter(lambda x: x.type == ph.UI_TYPE_ROADWAY, all_used_uimods), None),
            next(filter(lambda x: x.type == ph.UI_TYPE_LOCATION, all_used_uimods), None))

        (spotspeed_uimods, redlight_uimods, rc_uimods) = (
            [x for x in all_used_uimods if x.type == ph.UI_TYPE_SPOT_SPEED],
            [x for x in all_used_uimods if x.type == ph.UI_TYPE_RED_LIGHT],
            [x for x in all_used_uimods if x.type == ph.UI_TYPE_RAILROAD_CROSSING],
        )

        spotspeed_uimod = None
        if len(spotspeed_uimods) == 1:
            spotspeed_uimod = spotspeed_uimods[0]

        if entry_uimod.type == ph.UI_TYPE_HTTP_STREAMING:
            streaming_uimod = hstreaming_uimod

        elif entry_uimod.type == ph.UI_TYPE_RTSP_STREAMING:
            streaming_uimod = rstreaming_uimod

        else:
            raise NotImplementedError

        activate = any([
            uimod_to_newconf_map.get(streaming_uimod),
            uimod_to_newconf_map.get(gnss_uimod),
            uimod_to_newconf_map.get(cam_uimod),
            uimod_to_newconf_map.get(roadway_uimod),
            uimod_to_newconf_map.get(term_uimod),
            uimod_to_newconf_map.get(spotspeed_uimod),
            uimod_to_newconf_map.get(location_uimod),
            any([uimod_to_newconf_map.get(x) for x in redlight_uimods]),
            any([uimod_to_newconf_map.get(x) for x in rc_uimods]),
        ])

        if forced or activate:
            creator = _Creator(
                streaming_uimod, fusion_uimod, gnss_uimod, roadway_uimod, term_uimod, location_uimod,
                redlight_uimods, rc_uimods, cam_uimod,
                webserver_uimod=ws_uimod, radar_uimod=ra_uimod, spotspeed_uimod=spotspeed_uimod)

            return [creator]

    except StopIteration:
        pass

    return []


class _Creator(BaseCreator):
    def __init__(self,
                 streaming_uimod: ph.UiMod,
                 fusion_uimod: ph.UiMod,
                 gnss_uimod: ph.UiMod,
                 roadway_uimod: ph.UiMod,
                 terminal_uimod: ph.UiMod,
                 location_uimod: Optional[ph.UiMod],
                 redlight_uimods: List[ph.UiMod],
                 railroad_uimods: List[ph.UiMod],
                 cam_uimod: Optional[ph.UiMod] = None,
                 webserver_uimod: Optional[ph.UiMod] = None,
                 radar_uimod: Optional[ph.UiMod] = None,
                 spotspeed_uimod: Optional[ph.UiMod] = None):

        super().__init__()

        self._dgmod_rn: Final = dg.DgMod.create(dg.EDGE_TYPE_RENDERER, streaming_uimod.pid, streaming_uimod.id)

        self._uimod_camera: Final = cam_uimod
        self._uimod_fusion: Final = fusion_uimod
        self._uimod_gnss: Final = gnss_uimod
        self._uimod_location: Final = location_uimod
        self._uimod_radar: Final = radar_uimod
        self._uimod_roadway: Final = roadway_uimod
        self._uimod_spotspeed: Final = spotspeed_uimod
        self._uimod_streaming: Final = streaming_uimod
        self._uimod_terminal: Final = terminal_uimod
        self._uimod_webserver: Final = webserver_uimod

        self._uimods_railroad: Final = railroad_uimods
        self._uimods_redlight: Final = redlight_uimods

        self._main_ss = None

    @property
    def dgmod(self) -> dg.DgMod:
        return self._dgmod_rn

    def prepare(self, uimod_to_newconf_map: UiModToConfMap) -> None:
        ss_builder = SetupStoreConfBuilder()
        main_builder = MainConfBuilder(self._uimod_streaming.id, ph.get_spool_dir(self._uimod_streaming.pid))
        main_builder.setup_service_origin(ph.TermAboutUiconf.get_conf_path(self._uimod_streaming.pid))
        main_builder.setup_service_datetime(TimeUiconf.CONF_PATH)

        providers = list[IUiConfProvider]()

        uimod = self._uimod_roadway
        providers.extend([
            FusionUiConfProvider(self._uimod_fusion.id),
            GnssUiConfProvider(self._uimod_gnss.id, self._uimod_gnss.conf_path),
            RoadwayUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
        ])

        if self._uimod_streaming.type == ph.UI_TYPE_HTTP_STREAMING:
            uimod = self._uimod_streaming
            providers.extend([
                HttpStreamingUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf()),
                WebServerUiConfProvider(self._uimod_webserver.id)
            ])

        elif self._uimod_streaming.type == ph.UI_TYPE_RTSP_STREAMING:
            uimod = self._uimod_streaming
            providers.extend([
                RtspStreamingUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
            ])

        else:
            raise NotImplementedError()

        if self._uimod_radar:
            main_builder.setup_image_anno__ra_enabled(True)

        if self._uimod_camera:
            uimod = self._uimod_camera
            providers.extend([
                CameraUiConfProvider(uimod.id, uimod.conf_path)
            ])

        if self._uimod_location:
            uimod = self._uimod_location
            providers.extend([
                LocationUiConfProvider(uimod.id, uimod.conf_path)
            ])

        if self._uimod_spotspeed is not None:
            uimod = self._uimod_spotspeed
            providers.extend([
                SpotSpeedUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
            ])

        if len(self._uimods_redlight) == 1:
            uimod = self._uimods_redlight[0]
            providers.extend([
                RedlightUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
            ])

        elif len(self._uimods_railroad) == 1:
            uimod = self._uimods_railroad[0]
            providers.extend([
                RailroadUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
            ])

        for provider in providers:
            provider.setup_builders(main_builder, ss_builder)

        main_conf = main_builder.build()
        ss_active = ss_builder.build()

        self._main_ss = main_conf, ss_active

    def complete(self) -> None:
        main_conf, ss_active = self._main_ss  # type: dict, bool

        dgmod = self._dgmod_rn
        dgmod.install()
        dgmod.save_main_conf(main_conf)
        dgmod.save_setupstore_conf(ss_active)

    def rollback(self) -> None:
        pass

    def notify(self) -> None:
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class OrDeleter:
    def __init__(self, dgmod: dg.DgMod):
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod: ph.UiMod) -> Optional[Self]:
        if entry_uimod.type in [ph.UI_TYPE_HTTP_STREAMING, ph.UI_TYPE_RTSP_STREAMING]:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_RENDERER, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self) -> None:
        self._dgmod.uninstall()
