from typing import List, Optional, Self, Set, Union

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator, UiModToConfMap
from edge.webserver.sysconf.dgmod.rs.rs_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.rs.rs_setupstore import SetupStoreConfBuilder
from edge.webserver.sysconf.dgmod.rs.uiconf import IUiConfProvider
from edge.webserver.sysconf.dgmod.rs.uiconf.camera import MvCameragUiConfProvider
from edge.webserver.sysconf.dgmod.rs.uiconf.streaming import RtspStreamingUiConfProvider


# noinspection PyUnusedLocal
def rs_get_creators(entry_uimod: Union[ph.UiMod, ph.UiMod2],
                    all_used_uimods: Set[Union[ph.UiMod, ph.UiMod2]],
                    uimod_to_newconf_map: UiModToConfMap,
                    forced: bool = False) -> List[BaseCreator]:

    if entry_uimod.type != ph.UI_TYPE_RTSP_STREAMING:
        return []

    try:
        cam_uimod, rstreaming_uimod = (
            next(filter(lambda x: x.type == ph.UI_TYPE_MV_CAMERA, all_used_uimods), None),
            next(filter(lambda x: x.type == ph.UI_TYPE_RTSP_STREAMING, all_used_uimods)))

        activate = any([
            uimod_to_newconf_map.get(cam_uimod),
            uimod_to_newconf_map.get(rstreaming_uimod)
        ])

        if forced or activate:
            return [RsCreator(rstreaming_uimod, cam_uimod)]

    except StopIteration:
        pass

    return []


class RsCreator(BaseCreator):
    def __init__(self, rs_uimod: ph.UiMod, cam_uimod: ph.UiMod):
        super().__init__()

        self._dgmod_rs = dg.DgMod.create(dg.EDGE_TYPE_RTSP_SERVER, rs_uimod.pid, rs_uimod.id)

        self._uimod_rs = rs_uimod
        self._uimod_cam = cam_uimod

        self._info_main_ss = None

    @property
    def dgmod(self) -> dg.DgMod:
        return self._dgmod_rs

    def prepare(self, uimod_to_newconf_map: UiModToConfMap) -> None:
        ss_builder = SetupStoreConfBuilder()
        main_builder = MainConfBuilder(self._uimod_rs.id, ph.get_spool_dir(self._uimod_rs.pid))

        providers = list[IUiConfProvider]()

        uimod = self._uimod_rs
        providers.extend([
            RtspStreamingUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
        ])

        uimod = self._uimod_cam
        providers.extend([
            MvCameragUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
        ])

        for provider in providers:
            provider.setup_builders(main_builder, ss_builder)

        main_conf = main_builder.build()
        ss_active = ss_builder.build()

        self._info_main_ss = main_conf, ss_active

    def complete(self) -> None:
        main_conf, ss_active = self._info_main_ss  # type: dict, bool

        dgmod = self._dgmod_rs
        dgmod.install()
        dgmod.save_main_conf(main_conf)
        dgmod.save_setupstore_conf(ss_active)

    def rollback(self) -> None:
        pass

    def notify(self) -> None:
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class RsDeleter:
    def __init__(self, dgmod: dg.DgMod):
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod: ph.UiMod) -> Optional[Self]:
        if entry_uimod.type == ph.UI_TYPE_RTSP_STREAMING:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_RTSP_SERVER, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self) -> None:
        self._dgmod.uninstall()
