from typing import List, Optional, Self, Set, Union

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator, UiModToConfMap
from edge.webserver.sysconf.dgmod.tl.tl_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.tl.uiconf import IUiConfProvider
from edge.webserver.sysconf.dgmod.tl.uiconf.camera import CameraUiConfProvider
from edge.webserver.sysconf.dgmod.tl.uiconf.railroad import RailroadUiConfProvider
from edge.webserver.sysconf.dgmod.tl.uiconf.redlight import RedlightUiConfProvider


# noinspection PyUnusedLocal
def tl_get_creators(entry_uimod: Union[ph.UiMod, ph.UiMod2],
                    all_used_uimods: Set[Union[ph.UiMod, ph.UiMod2]],
                    uimod_to_newconf_map: UiModToConfMap,
                    forced: bool = False) -> List[BaseCreator]:

    if entry_uimod.type != ph.UI_TYPE_TRAFFIC_LIGHT:
        return []

    try:
        cam_uimod, tl_uimod, rules_uimod, redlight_uimods, railroad_crossing_uimods = [
            next(filter(lambda x: x.type in [ph.UI_TYPE_MV_CAMERA, ph.UI_TYPE_RTSP_CAMERA], all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_TRAFFIC_LIGHT, all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_TRAFFIC_RULES, all_used_uimods)),
            [x for x in all_used_uimods if x.type == ph.UI_TYPE_RED_LIGHT],
            [x for x in all_used_uimods if x.type == ph.UI_TYPE_RAILROAD_CROSSING]]

        activate = any([uimod_to_newconf_map.get(x) for x in redlight_uimods]) or len(redlight_uimods) == 0 or \
                   (any([uimod_to_newconf_map.get(x) for x in railroad_crossing_uimods]) or len(railroad_crossing_uimods) == 0)
        if forced or activate:
            return [TlCreator(cam_uimod, tl_uimod, rules_uimod, redlight_uimods, railroad_crossing_uimods)]

    except StopIteration:
        pass

    return []


class TlCreator(BaseCreator):
    def __init__(self,
                 cam_uimod: ph.UiMod,
                 tl_uimod: ph.UiMod,
                 rules_uimod: ph.UiMod,
                 redlight_uimods: List[ph.UiMod],
                 railroad_uimods: List[ph.UiMod]) -> None:

        super().__init__()

        self._dgmod_tl = dg.DgMod.create(dg.EDGE_TYPE_TRAFFIC_LIGHT, tl_uimod.pid, tl_uimod.id)

        self._uimod_cam = cam_uimod
        self._uimod_tl = tl_uimod
        self._uimod_rules = rules_uimod
        self._uimods_redlight = redlight_uimods
        self._uimods_railroad = railroad_uimods

        self._main_ss = None

    @property
    def dgmod(self) -> dg.DgMod:
        return self._dgmod_tl

    def prepare(self, uimod_to_newconf_map: UiModToConfMap) -> None:
        main_builder = MainConfBuilder(self._uimod_tl.id, ph.get_spool_dir(self._uimod_tl.pid))
        main_builder.setup_track_event_args([
            f'tl{main_builder.uimod_id}-sc{self._uimod_rules.id}-track',
            # f'tl{main_builder.uimod_id}-or{self._rules_uimod.id}-track'
        ])

        providers = list[IUiConfProvider]()

        if len(self._uimods_redlight) == 1:
            rl_uimod = self._uimods_redlight[0]
            rl_uiconf = uimod_to_newconf_map.get(rl_uimod) or rl_uimod.load_conf()
            rl_uiconf_pri = rl_uiconf['ro']

            with_trafficlight_65535 = rl_uiconf_pri.get('with-trafficlight.65535', False)
            with_trafficlight = not with_trafficlight_65535

            providers.extend([
                CameraUiConfProvider(self._uimod_cam.id),
                RedlightUiConfProvider(rl_uimod.id, rl_uiconf)
            ])

        elif len(self._uimods_railroad) == 1:
            rc_uimod = self._uimods_railroad[0]
            rc_uiconf = uimod_to_newconf_map.get(rc_uimod) or rc_uimod.load_conf()
            rc_uiconf_pri = rc_uiconf['ro']

            with_trafficlight_65535 = rc_uiconf_pri.get('with-trafficlight.65535', False)
            with_trafficlight = not with_trafficlight_65535

            providers.extend([
                CameraUiConfProvider(self._uimod_cam.id),
                RailroadUiConfProvider(rc_uimod.id, rc_uiconf)
            ])

        else:
            with_trafficlight = False

        for provider in providers:
            provider.setup_builders(main_builder)

        main_conf = main_builder.build()

        self._main_ss = main_conf, with_trafficlight

    def complete(self) -> None:
        main_conf, ss_active = self._main_ss  # type: dict, bool

        dgmod = self._dgmod_tl
        dgmod.install()
        dgmod.save_main_conf(main_conf)
        dgmod.save_setupstore_conf(ss_active)

    def rollback(self) -> None:
        pass

    def notify(self) -> None:
        dg.LifeCycle.sighup_ss_dgmod()


class TlDeleter:
    def __init__(self, dgmod: dg.DgMod):
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod: ph.UiMod) -> Optional[Self]:
        if entry_uimod.type == ph.UI_TYPE_TRAFFIC_LIGHT:
            dgmod = dg.DgMod.create(
                dg.EDGE_TYPE_TRAFFIC_LIGHT, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self) -> None:
        self._dgmod.uninstall()
